@@ -1,10 +1,17 @@
 import functools
+from enum import Enum
 
 from sgraph import SElement
 from sgraph import SElementAssociation
 from sgraph import SGraph
 
 
+class FilterAssocations(Enum):
+    Ignore = 1
+    Direct = 2
+    DirectAndIndirect = 3
+
+
 class ModelApi:
     def __init__(self, filepath=None, model=None):
         if filepath:
@@ -72,8 +79,8 @@ class ModelApi:
                         if prevent_self_deps and self.matches_with_descendant(ea.toElement,
                                                                               tuple([f])):
                             pass
-                        elif ea.toElement in to_elems or self.matches_with_descendant(
-                                ea.toElement, tuple(to_elems)):
+                        elif ea.toElement in to_elems or self.matches_with_descendant(ea.toElement,
+                                tuple(to_elems)):
                             self.add_if_matches(ea, found, dep_filter)
 
                 for c in elem.children:
@@ -148,29 +155,94 @@ class ModelApi:
             if not ModelApi.intra_file(ea):
                 found.append(ea)
 
-    def filter_model(self, source_elem, source_graph):
+    def filter_model(self, source_elem, source_graph,
+                     filter_outgoing: FilterAssocations = FilterAssocations.Direct,
+                     filter_incoming: FilterAssocations = FilterAssocations.Direct):
+        """
+        Filter a sub graph from source_graph related to source elem.
+
+        When executing model_filter for elem e with "Ignore" mode, it ignores elements
+        that are external to e.
+
+        "Direct" mode changes this behavior: it picks also external elements associated
+        with descendants of e. However, it is limited to direct assocations.
+
+        "DirectAndIndirect" mode selects all directly and indirectly associated elements
+        and their descendants.
+
+        Traversal of "DirectAndIndirect" is not directed, i.e. it goes in undirected mode,
+        leading into situations where you have the dependencies of your users also included.
+
+        :param source_elem: element for filtering (descendants of it)
+        :param source_graph: the overall graph to be filtered
+        :param filter_outgoing: filtering mode for outgoing dependencies
+        :param filter_incoming: filtering mode for incoming dependencies
+        :return: new graph, that contains independent new SElement objects with same topology as
+        in the source_graph
+        """
         sub_graph = SGraph()
         sub_graph.createOrGetElement(source_elem)
         stack = [source_elem]
+
+        def is_descendant_of_source_elem(element):
+            ancestor = element
+            while ancestor is not None and ancestor.parent is not None:
+                ancestor = ancestor.parent
+                if ancestor == source_elem:
+                    return True
+
+        def create_assoc(x, other, is_outgoing, ea):
+            new_or_existing_referred_elem, is_new = sub_graph.create_or_get_element(x)
+
+            if is_outgoing:
+                SElementAssociation(other, new_or_existing_referred_elem, ea.deptype,
+                                    ea.attrs).initElems()
+            else:
+
+                SElementAssociation(new_or_existing_referred_elem, other, ea.deptype,
+                                    ea.attrs).initElems()
+
+            return new_or_existing_referred_elem, is_new
+
+        def handle_assocation(ea, related_elem, filter_setting, is_outgoing, stack):
+            descendant_of_src = is_descendant_of_source_elem(related_elem)
+
+            if not descendant_of_src and filter_setting == FilterAssocations.Ignore:
+                return
+
+            new_or_existing_referred_elem, is_new = create_assoc(related_elem, new_elem,
+                                                                 is_outgoing, ea)
+
+            if descendant_of_src:
+                # No need to create descendants since those will be anyway created later as part of
+                # the main iteration.
+                return
+            elif filter_setting == FilterAssocations.Direct:
+                if is_new:
+                    # Avoid creating descendants multiple times.
+                    self.create_descendants(related_elem, new_or_existing_referred_elem)
+
+            elif filter_setting == FilterAssocations.DirectAndIndirect:
+                # Get all indirectly and directly used elements into the subgraph, including
+                # their descendant elements.
+                if ea.toElement not in handled:
+                    stack.append(ea.toElement)
+
+        handled = set()
         # Traverse related elements from the source_graph using stack
         while stack:
             elem = stack.pop(0)
+            handled.add(elem)
+
             new_elem = sub_graph.createOrGetElement(elem)
             new_elem.attrs = elem.attrs
+
             for ea in elem.outgoing:
-                selem_from = new_elem
-                to_element = ea.toElement
-                selem_to = sub_graph.createOrGetElement(to_element)
-                sea = SElementAssociation(selem_from, selem_to, ea.deptype, ea.attrs)
-                sea.initElems()
-                if len(to_element.children) > 0:
-                    for child in to_element.children:
-                        sub_graph.createOrGetElement(child)
+                handle_assocation(ea, ea.toElement, filter_outgoing, True, stack)
+
             for ea in elem.incoming:
-                selem_from = sub_graph.createOrGetElement(ea.fromElement)
-                selem_to = new_elem
-                sea = SElementAssociation(selem_from, selem_to, ea.deptype, ea.attrs)
-                sea.initElems()
+                handle_assocation(ea, ea.fromElement, filter_incoming, False, stack)
+
             stack.extend(elem.children)
 
         # Now that elements have been created, copy attribute data from the whole graph, via
@@ -186,3 +258,14 @@ class ModelApi:
                 whole_graph_stack.append(corresponding_source_elem.getChildByName(elem.name))
 
         return sub_graph
+
+    def create_descendants(self, related_elem: SElement, new_or_existing_referred_elem: SElement):
+
+        stack = [(related_elem, new_or_existing_referred_elem)]
+        while stack:
+            orig, new = stack.pop(0)
+
+            if orig.children:
+                for child in orig.children:
+                    new_child = new.createOrGetElement(child.name)
+                    stack.append((child, new_child))
